@@ -23,4 +23,9 @@ class AnonymousProfile(object):
     is_public = False
 
     def is_attached_to_user(self, user):
-        return False
\ No newline at end of file
+        return False
+
+
+class TestObject(models.Model):
+
+    title = models.CharField(max_length=50)
\ No newline at end of file
